<?php

namespace Drupal\remote_file_importer;

use Drupal\remote_file_importer\Entity\DataSource;

/**
 * Manages Batch functionalities for import.
 */
class ImportBatch {

  /**
   * Process the batch.
   *
   * @param string $dataSourcePluginId
   *   Id of f data source plugin.
   * @param array $items
   *   Array of items to be imported.
   *   Each item is an array with keys: local_file, remote_file.
   * @param array $context
   *   Batch context.
   */
  public static function process($dataSourcePluginId, $items, &$context) {
    $pluginManager = \Drupal::service('plugin.manager.remote_file_importer.data_source');
    $dataSource = \Drupal::entityTypeManager()->getStorage('rfi_data_source')->load($dataSourcePluginId);
    $dataSourcePlugin = $pluginManager->createInstance($dataSource->plugin_id);

    if ($dataSourcePlugin->establishConnection($dataSource->settings)) {
      foreach ($items as $item) {
        $dataSourcePlugin->importFile($item['local_file'], $item['remote_file']);
        if ($dataSource->keep_modify) {
          $time = strtotime($item['metadata']['modified_date']);
          touch($item['local_file'], $time);
        }

        $context['results'][] = $item['local_file'];
      }
      $dataSourcePlugin->closeConnection();
    }

  }

  /**
   * Clean up remote deleted files.
   *
   * @param string $source_folder
   *   The datasource destination folder.
   * @param array $files
   *   Array of files to be deleted.
   * @param array $context
   *   Batch context.
   */
  public static function cleanUp(string $source_folder, $files, &$context) {
    $fileSystem = \Drupal::service('file_system');
    foreach ($files as $file) {
      // Remove files that are not in remote folder.
      $file = $fileSystem->delete($file);
    }

    // Remove empty folders.
    $cleanUpService = \Drupal::service('remote_file_importer.cleanup_service');
    $cleanUpService->deleteEmptySubfolders($fileSystem->realpath($source_folder));
  }

  /**
   * Called when the batch is finished.
   *
   * @param bool $success
   *   Whether the batch finished successfully.
   * @param array $results
   *   Array of results.
   * @param array $operations
   *   Array of operations.
   * @param float $elapsed
   *   Time elapsed.
   */
  public static function finished($success, $results, $operations, $elapsed) {
    if ($success) {
      $message = \Drupal::translation()
        ->formatPlural(count($results), 'One file imported.', '@count files imported.');
    }
    else {
      $message = t('Finished with an error.');
    }
    \Drupal::messenger()
      ->addMessage($message);
  }

}
